;***************************************************************************
;*   Library Name: Player
;*       Coded By: John Logsdon
;*   Last Updated: 8/5/2012
;*        Purpose: Handles all of the player procedures
;***************************************************************************


;*************************************************************
; PROCEDURE: LoadPlayerFrames(...)
;
; This function moves the player's character by a certain
; number of pixels.
;
; Arguments: 
;   Player_Full_Path: File name, including full path
;   PlayerWidth:  How wide the image is
;   PlayerHeight: how high the image is
;   PlayerSpacer: how many pixels between each image in the file
;
; Returns:
;   0 = Everything loaded fine
;   ? = Load failed
;*************************************************************
Procedure LoadPlayerFrames(Player_Full_Path.s,PlayerWidth.l,PlayerHeight.l,PlayerSpacer.l)

  Dim Player.Player_Frame_Images(0)
  
  ; first off let's load the full image containing all the tiles
  ; into a temporary space
  Temp_Image.l = LoadSprite(#PB_Any,Player_Full_Path.s)
  
  ImageSizeX.l = SpriteWidth(Temp_Image.l)
  ImageSizeY.l = SpriteHeight(Temp_Image.l)
  
  PlayerColumns.l = ImageSizeX.l / PlayerWidth.l
  PlayerRows.l = ImageSizeY.l / PlayerHeight.l
  
  Dim Player.Player_Frame_Images(PlayerRows.l * PlayerColumns.l)
  
  If Temp_Image.l = 0
    ProcedureReturn(-1)
  EndIf
  
  ; get the current buffer so we can restore to it
  DisplaySprite(Temp_Image,0,0)

  ; setup our basic vars.  The X and Y values will be whatever the XSpacer
  ; and YSpacer values are. If they are 0, then it's assumed that there is
  ; no space between the tiles. If you look at the sample images included
  ; with this demo you'll see a white box around each image...that's why
  ; I have these spacers...we don't wanna load the boxes, just the images.
  X.l = PlayerSpacer.l
  Y.l = PlayerSpacer.l
  
  ; keep track of the number of images
  ImageNumber.l = 0
  
  ; run through the total number of rows (minus 1, of course)
  For Rows.l = 0 To PlayerRows.l -1
    ; and run through all the columns per row (minus 1 again)
    For Columns.l = 0 To PlayerColumns.l -1
      ; create a new TileList element and assign it the current
      ; ImageNumber.  Then populate the TileList element with
      ; the TileWidth, TileHeight, and the actual Tile_Image
      Player(ImageNumber)\Width = PlayerWidth
      Player(ImageNumber)\Height = PlayerHeight
      
      ; then we grab the image based off the size setup in CreateImage(...)
      ; from our X,Y location
      Player(ImageNumber)\Image=GrabSprite(#PB_Any,X,Y,PlayerWidth,PlayerHeight)
      
      ; now we add X to the TileWidth and the XSpacer to get the new X position.
      ; So, if our current X = 2, and the TileWidth = 32 and the XSpacer = 2, we'd
      ; have X = 2 + 32 + 2, or 36.  This means that the next time we call         
      ; GrabImage(...) it will start grabbing from the X position 36 (or the 36th pixel 
      ; from the left).         
      X.l = X.l + PlayerWidth.l + PlayerSpacer.l
      
      ; increment our tile counter (for the array positioning)
      ImageNumber.l = ImageNumber.l + 1
    Next
    ; we've finished with that row, so reset X back to the spacer position
    X.l = PlayerSpacer.l
    
    ; now we add Y to the TileHeight and the YSpacer to get the new Y position.
    ; So, if our current Y = 2, and the TileHeight = 32 and the YSpacer = 2, we'd
    ; have Y = 2 + 32 + 2, or 36.  This means that the next time we call  
    ; GrabImage(...) it will start grabbing from the Y position 36 (or the 36th pixel 
    ; from the top).         
    Y.l = Y.l + PlayerHeight.l + PlayerSpacer.l
  Next
  
  ; free the image from memory, so we don't hold memory for no reason
  FreeSprite(Temp_Image)
  
  ProcedureReturn(0)
EndProcedure



;*************************************************************
; PROCEDURE: MovePlayer(...)
;
; This function moves the player's character by a certain
; number of pixels.
;
; Arguments: 
;   Direction: 0=Not Moving, 1=up,2=down,3=left,4=right
;   Distance: how many pixels to move?
;   TileWidth: how wide are the tiles
;   TileHeight: how high are the tiles
;
; Returns:
;   0 = Don't scroll the map
;   1 = Scroll the map
;*************************************************************
Procedure MovePlayer(Direction.l,Distance.l,TileWidth,TileHeight)

  MoveMapX.l = 0
  MoveMapY.l = 0
  Select Direction.l
    ; Standing still
    Case 0
      ; If we're at an edge, let the sprite walk independently of the scroll
      If PlayerWorldX <= ScreenX Or PlayerWorldX > Map_Pixel_Width - ScreenX
        If PlayerWorldX <= ScreenX
          DrawPlayerX = PlayerWorldX
        Else
          DrawPlayerX = ViewPortWidth - (Map_Width * TileWidth - PlayerWorldX)
        EndIf
      Else
        ; We must not be at an edge, so we want to keep the sprite in the middle
        ; and let the program know we're good to scroll
        DrawPlayerX = ScreenX
        MoveMapX.l = 1
      EndIf
    
      ; If we're at an edge, let the sprite walk independently of the scroll
      If PlayerWorldY <= ScreenY Or PlayerWorldY > Map_Pixel_Height - ScreenY
        If PlayerWorldY <= ScreenY
          DrawPlayerY = PlayerWorldY
        Else
          DrawPlayerY = ViewPortHeight - (Map_Height * TileHeight - PlayerWorldY)
        EndIf
      Else
        ; We must not be at an edge, so we want to keep the sprite in the middle
        ; and let the program know we're good to scroll
        DrawPlayerY = ScreenY
        MoveMapY.l = 1
      EndIf
    
      ; Going Up
    Case 1
      ; save the current World value
      OldY.l = PlayerWorldY
      ; calculate the new value based on the distance
      PlayerWorldY = PlayerWorldY - Distance.l 
      ; make sure the new value is >= 0
      If PlayerWorldY < 0
        PlayerWorldY = 0
      EndIf
      
      ; See if there's a collision
      If Map_CheckWallCollision(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight) = 1
        ; If so, reset the PlayerWorldY value
        PlayerWorldY = OldY.l
      Else
        ; Otherwise, see where we are on the map
        ; If we're at an edge, let the sprite walk independently of the scroll
        If PlayerWorldY < ScreenY Or PlayerWorldY >= Map_Pixel_Height - ScreenY
          If PlayerWorldY <= ScreenY
            DrawPlayerY = PlayerWorldY
          Else
            DrawPlayerY = ViewPortHeight - (Map_Height * TileHeight - PlayerWorldY)
          EndIf
        Else 
          ; We must not be at an edge, so we want to keep the sprite in the middle
          ; and let the program know we're good to scroll
          DrawPlayerY = ScreenY
          MoveMapY.l = 1
        EndIf
      EndIf
    
    ; Going Down
    Case 2
      OldY.l = PlayerWorldY
      PlayerWorldY = PlayerWorldY + Distance.l
      If PlayerWorldY > Map_Height * TileHeight - TileHeight
        PlayerWorldY = Map_Height * TileHeight - TileHeight
      EndIf
      ; See if there's a collision
      If Map_CheckWallCollision(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight) = 1
        ; If so, reset the PlayerWorldY value
        PlayerWorldY = OldY.l
      Else
        ; Otherwise, see where we are on the map
        ; If we're at an edge, let the sprite walk independently of the scroll
        If PlayerWorldY <= ScreenY Or PlayerWorldY > Map_Pixel_Height - ScreenY
          If PlayerWorldY <= ScreenY
            DrawPlayerY = PlayerWorldY
          Else
            DrawPlayerY = ViewPortHeight - (Map_Height * TileHeight - PlayerWorldY)
          EndIf
        Else
          ; We must not be at an edge, so we want to keep the sprite in the middle
          ; and let the program know we're good to scroll
          DrawPlayerY = ScreenY
          MoveMapY.l = 1
        EndIf
      EndIf
    
    ; Going Left
    Case 3
      OldX.l = PlayerWorldX
      PlayerWorldX = PlayerWorldX - Distance.l 
      If PlayerWorldX < 0
        PlayerWorldX = 0
      EndIf
      ; See if there's a collision
      If Map_CheckWallCollision(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight) = 1
        ; If so, reset the PlayerWorldX value
        PlayerWorldX = OldX.l
      Else
        ; Otherwise, see where we are on the map
        ; If we're at an edge, let the sprite walk independently of the scroll
        If PlayerWorldX < ScreenX Or PlayerWorldX >= Map_Pixel_Width - ScreenX
          If PlayerWorldX < ScreenX
            DrawPlayerX = PlayerWorldX
          Else
            DrawPlayerX = ViewPortWidth - (Map_Width * TileWidth - PlayerWorldX)
          EndIf
        Else
          ; We must not be at an edge, so we want to keep the sprite in the middle
          ; and let the program know we're good to scroll
          DrawPlayerX = ScreenX
          MoveMapX.l = 1
        EndIf
      EndIf
    
    ; Going Right
    Case 4
      OldX.l = PlayerWorldX
      PlayerWorldX = PlayerWorldX + Distance.l 
      If PlayerWorldX > Map_Width * TileWidth - TileWidth
        PlayerWorldX = Map_Width * TileWidth - TileWidth 
      EndIf
      ; See if there's a collision
      If Map_CheckWallCollision(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight) = 1
        ; If so, reset the PlayerWorldX value
        PlayerWorldX = OldX.l
      Else
        ; Otherwise, see where we are on the map
        ; If we're at an edge, let the sprite walk independently of the scroll
        If PlayerWorldX <= ScreenX Or PlayerWorldX > Map_Pixel_Width - ScreenX
          If PlayerWorldX <= ScreenX
            DrawPlayerX = PlayerWorldX
          Else
            DrawPlayerX = ViewPortWidth - (Map_Width * TileWidth - PlayerWorldX)
          EndIf
        Else
          ; We must not be at an edge, so we want to keep the sprite in the middle
          ; and let the program know we're good to scroll
          DrawPlayerX = ScreenX
          MoveMapX.l = 1
        EndIf
      EndIf
  EndSelect

  ; if either X or Y is scrollable, return 1 to let the caller know
  If MoveMapX.l = 1 Or MoveMapY.l = 1
    ProcedureReturn(1)
  Else
    ProcedureReturn(0)
  EndIf
EndProcedure



;*************************************************************
; PROCEDURE: AnimatePlayer(...)
;
; This function sets the proper frame for the next draw
;
; Arguments: 
;   Current_Time: last call to elapsedmilliseconds()
;   Animation_Time: how fast to animate
;   Direction: which way is the player facing?
;   TileWidth: how wide are the images
;   TileHeight: how high are the images
;
; Returns:
;   N/A
;*************************************************************
Procedure AnimatePlayer(Current_Time,Animation_Time.l,Direction.l,TileWidth,TileHeight)
  ; if the player is neither hit nor dead
  If PlayerHitAnimation = 0 And PlayerDead = 0
     ; if the player's not firing
     If PlayerFiring = 0
        ; if direction is 0 show the standing still frame of the player (based on direction facing)
        If Direction.l = 0
          Select PlayerFacing
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;     FACING UP
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            Case 1        
              ; if he's on a ladder, keep him climbing  
              MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight)
              If MTile = 1
                PlayerFrame = 87
              Else ; otherwise just have him look to the right
                PlayerFrame = 40
              EndIf

            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;     FACING DOWN
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            Case 2          
              MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight)
              ; if he's on a ladder, keep him climbing  
              If MTile = 1
                PlayerFrame = 87
              Else ; otherwise just have him look to the right
                PlayerFrame = 40
              EndIf
              
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;     FACING LEFT
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            Case 3              
            ; make sure the player's not fidgeting
            If Current_Time > Fidget_Timer + Fidget_Interval And PlayerFidgeting = 0
              ; up the fidget level a bit...player's getting bored!
              Fidget_Level = Fidget_Level + 1
              If Fidget_Level > 7
                Fidget_Level = 7
              EndIf
              ; do a little fidget move
              If Fidget_Level > 0 And Fidget_Level < 3
                Fidget_Speed = 250
                PlayerFrame = 7
                Fidget_Interval = 2000
              EndIf
              ; yawn a bit
              If Fidget_Level > 2 And Fidget_Level < 5
                Fidget_Speed = 500
                PlayerFrame = 0
                Fidget_Timer = Current_Time
                Fidget_Interval = 2000
              EndIf
              ; yawn a smidgeon longer
              If Fidget_Level > 4 And Fidget_Level < 7
                Fidget_Speed = 1250
                PlayerFrame = 0
                Fidget_Interval = 2000
              EndIf
              ; okay, that's it...you're asleep
              If Fidget_Level >= 7
                Fidget_Speed = 250
                PlayerFrame = 111
                Fidget_Interval = 100
                PlayerSleeping = 1
              EndIf
              ; player is definitely fidgeting
              PlayerFidgeting = 1
            EndIf
      
              ; now we handle the animation
              ; The player character is fidgeting      
              If PlayerFidgeting = 1
                ; just do a little fidget, crack the knuckles
                If Fidget_Level > 0 And Fidget_Level < 3
                  ; assuming it's the right time of course
                  If Current_Time > Frame_Timer + Fidget_Speed
                    PlayerFrame = PlayerFrame - 1
                    If PlayerFrame < 3
                      PlayerFrame = 7
                      PlayerFidgeting = 0
                      ; reset the frame timer
                      Fidget_Timer = Current_Time
                    EndIf
                    ; reset the frame timer
                    Frame_Timer = Current_Time
                  EndIf              
                EndIf

                ; same as above
                If Fidget_Level > 2 And Fidget_Level < 5
                  If Current_Time > Frame_Timer + Fidget_Speed
                    PlayerFrame = 0
                    ; reset the frame timer
                    Fidget_Timer = Current_Time
                    ; reset the frame timer
                    Frame_Timer = Current_Time
                    PlayerFidgeting = 0
                  EndIf              
                EndIf
                
                ; same as above
                If Fidget_Level > 4 And Fidget_Level < 7
                  If Current_Time > Frame_Timer + Fidget_Speed
                    PlayerFrame = PlayerFrame + 1
                    If PlayerFrame > 1
                      PlayerFrame = 0
                      PlayerFidgeting = 0
                      ; reset the frame timer
                      Fidget_Timer = Current_Time
                    EndIf
                    ; reset the frame timer
                    Frame_Timer = Current_Time
                  EndIf              
                EndIf
                
              Else
                ; if the player is not fidgeting and not sleeping
                If Fidget_Level = 0 And PlayerSleeping = 0
                  ; just draw the player facing the appropriate direction
                  MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight)
                  If MTile = 0
                    PlayerFrame = 7
                  Else
                    PlayerFrame = 80
                  EndIf
                EndIf
              EndIf
      
              ; The player character is sleeping        
              If PlayerSleeping = 1
                If Current_Time > Snore_Timer + Snore_Interval
                  MyPlaySound(7)
                  Snore_Timer = Current_Time
                EndIf

                ; update the sleeping animation accordingly
                If Current_Time > Frame_Timer + Fidget_Speed
                  PlayerFrame = PlayerFrame - 1
                  If PlayerFrame < 104
                    PlayerFrame = 111
                  EndIf
                  ; reset the frame timer
                  Frame_Timer = Current_Time
                EndIf              
              EndIf
      
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ;     FACING RIGHT
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            Case 4  ; follow details from Case 3
              If Current_Time > Fidget_Timer + Fidget_Interval And PlayerFidgeting = 0
                Fidget_Level = Fidget_Level + 1
                If Fidget_Level > 7
                  Fidget_Level = 7
                EndIf
                If Fidget_Level > 0 And Fidget_Level < 3
                  Fidget_Speed = 250
                  PlayerFrame = 40
                  Fidget_Interval = 2000
                EndIf
                
                If Fidget_Level > 2 And Fidget_Level < 5
                  Fidget_Speed = 500
                  PlayerFrame = 47
                  Fidget_Timer = Current_Time
                  Fidget_Interval = 1000
                EndIf
                
                If Fidget_Level > 4 And Fidget_Level < 7
                  Fidget_Speed = 1250
                  PlayerFrame = 47
                  Fidget_Timer = Current_Time
                  Fidget_Interval = 1000
                EndIf
                
                If Fidget_Level >= 7
                  Fidget_Speed = 250
                  PlayerFrame = 96
                  Fidget_Interval = 100
                  PlayerSleeping = 1
                EndIf
                
                PlayerFidgeting = 1
              EndIf
              
              ; The player character is fidgeting        
              If PlayerFidgeting = 1
                If Fidget_Level > 0 And Fidget_Level < 3
                  If Current_Time > Frame_Timer + Fidget_Speed
                    PlayerFrame = PlayerFrame + 1
                    If PlayerFrame > 44
                      PlayerFrame = 40
                      PlayerFidgeting = 0
                      ; reset the frame timer
                      Fidget_Timer = Current_Time
                    EndIf
                    ; reset the frame timer
                    Frame_Timer = Current_Time
                  EndIf              
                EndIf
                
                If Fidget_Level > 2 And Fidget_Level < 5
                  If Current_Time > Frame_Timer + Fidget_Speed
                    PlayerFrame = 47
                    ; reset the frame timer
                    Fidget_Timer = Current_Time
                    ; reset the frame timer
                    Frame_Timer = Current_Time
                    PlayerFidgeting = 0
                  EndIf              
                EndIf
                
                If Fidget_Level > 4 And Fidget_Level < 7
                  If Current_Time > Frame_Timer + Fidget_Speed
                    PlayerFrame = PlayerFrame - 1
                    If PlayerFrame < 46
                      PlayerFrame = 47
                      PlayerFidgeting = 0
                      ; reset the frame timer
                      Fidget_Timer = Current_Time
                    EndIf
                    ; reset the frame timer
                    Frame_Timer = Current_Time
                  EndIf              
                EndIf
              Else
                If Fidget_Level = 0 And PlayerSleeping = 0
                  MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight)
                  If MTile = 0
                    PlayerFrame = 40
                  Else
                    PlayerFrame = 80
                  EndIf
                EndIf
              EndIf
      
; The player character is sleeping        
If PlayerSleeping = 1
  If Current_Time > Snore_Timer + Snore_Interval
    MyPlaySound(7)
    Snore_Timer = Current_Time
  EndIf

  If Current_Time > Frame_Timer + Fidget_Speed
    PlayerFrame = PlayerFrame + 1
    If PlayerFrame > 103
      PlayerFrame = 96
    EndIf
    ; reset the frame timer
    Frame_Timer = Current_Time
  EndIf              
EndIf
      
          EndSelect
        Else
          ; we're moving again, so let's make sure we reset the Fidget information
          Fidget_Level = 0
          Fidget_Timer = Current_Time
          PlayerFidgeting = 0
          Fidget_Interval = 5000
          Fidget_Speed = 250
          PlayerSleeping = 0
          
          ; is it time to update the player's frame?
          If Current_Time > Frame_Timer + Animation_Time
            ; increase the frame
            PlayerFrame = PlayerFrame + 1
            ; which way are we facing?
            Select PlayerFacing
              ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
              ;     FACING UP
              ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
              Case 1        
                ; are we on a ladder?
                MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight)
                ; yep...so show the climbing frames
                If MTile = 1
                  If PlayerFrame < 80
                      PlayerFrame = 80
                   EndIf
                   If PlayerFrame > 87
                    PlayerFrame = 80
                  EndIf
                Else ; nope, so show the regular static frame
                  PlayerFrame = 40
                EndIf

              ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
              ;     FACING DOWN
              ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
              Case 2  ; refer to Case 1         
                MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight)
                If MTile = 1
                  If PlayerFrame < 80
                    PlayerFrame = 80
                  EndIf
                  If PlayerFrame > 87
                    PlayerFrame = 80
                  EndIf
                Else
                  PlayerFrame = 40
                EndIf
                
              ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
              ;     FACING LEFT
              ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
              Case 3           
                ; are we on the ladder?
                MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight)
                ; yep, so just show the static frame (climbing)
                If MTile = 1
                  PlayerFrame = 80
                  PlayerFacing = 1
                Else ; nope, keep on moving
                  If PlayerFrame < 8
                     PlayerFrame = 8
                  EndIf
                  If PlayerFrame > 15
                    PlayerFrame = 8
                  EndIf
                EndIf

              ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
              ;     FACING RIGHT
              ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
              Case 4 ; refer to Case 3          
                MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,TileWidth,TileHeight)
                If MTile = 1
                  PlayerFrame = 80
                  PlayerFacing = 1
                Else
                  If PlayerFrame < 48
                    PlayerFrame = 48
                  EndIf
                  If PlayerFrame > 55
                    PlayerFrame = 48
                  EndIf
               EndIf
            EndSelect
            ; reset the frame timer
            Frame_Timer = Current_Time
          EndIf
        EndIf
     Else ; we're firing
        If Current_Time > Firing_Timer + Firing_Speed
           ; depending on which way the player was facing when firing, animate accordingly
           If PlayerFacing = 3
              PlayerFrame = PlayerFrame - 1
              If PlayerFrame > 31
                 PlayerFrame = 31
              EndIf
              ; when the frame hits the point where the laser should fly out, add the laser
              If PlayerFrame = 27
                 AddLaser(0,DrawPlayerX,DrawPlayerY+11,PlayerWorldX,PlayerWorldY,1,#TileWidth,#TileHeight)
              EndIf
              If PlayerFrame < 24
                 PlayerFrame = 31
                 PlayerFiring = 0
              EndIf
           Else
              PlayerFrame = PlayerFrame + 1
              If PlayerFrame < 64
                 PlayerFrame = 64
              EndIf
              ; when the frame hits the point where the laser should fly out, add the laser
              If PlayerFrame = 68
                 AddLaser(0,DrawPlayerX+32,DrawPlayerY+11,PlayerWorldX+32,PlayerWorldY,2,#TileWidth,#TileHeight)
              EndIf
              If PlayerFrame > 71
                 PlayerFrame = 64
                 PlayerFiring = 0
              EndIf
           EndIf
           Firing_Timer = Current_Time
        EndIf   
        ; make sure to reset all the fidget timers and stuff, since firing isn't consider "moving"
        Fidget_Level = 0
        Fidget_Timer = Current_Time
        PlayerFidgeting = 0
        Fidget_Interval = 5000
        Fidget_Speed = 250
        PlayerSleeping = 0
     EndIf
  Else ; Player's either been hit or killed
     ;Reset the fidgeting...getting shot'll wake you up, maybe.
     Fidget_Level = 0
     Fidget_Timer = Current_Time
     PlayerFidgeting = 0
     Fidget_Interval = 5000
     Fidget_Speed = 250
     PlayerSleeping = 0
     
     ; if the player is indeed dead
     If PlayerDead = 1
        ; start up the death animation and such and get the restart going for the level
        If PlayerFacing = 1 Or PlayerFacing = 3
           PlayerFrame = PlayerFrame - 1
           If PlayerFrame < 92
              PlayerDead = -1
           EndIf
        Else
           PlayerFrame = PlayerFrame + 1
           If PlayerFrame > 91
              PlayerDead = -1
           EndIf
        EndIf
     Else ; otherwise, the player has just been hit, so play that animation
        If PlayerFacing = 1 Or PlayerFacing = 3
           PlayerFrame = 95
           PlayerHitAnimation = PlayerHitAnimation - 1
        Else
           PlayerFrame = 88
           PlayerHitAnimation = PlayerHitAnimation - 1
        EndIf
     EndIf
     ; reset the frame timer
     Frame_Timer = Current_Time
  EndIf
EndProcedure



; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 533
; FirstLine = 519
; Folding = -